changeset 23921:a834f9c082e3 v8.2.2503

patch 8.2.2503: Vim9: a caught error may leave something on the stack Commit: https://github.com/vim/vim/commit/d9d7789b6fe5f2b4074375ee9f1c0bad3e4d3cfe Author: Bram Moolenaar <Bram@vim.org> Date: Fri Feb 12 21:32:47 2021 +0100 patch 8.2.2503: Vim9: a caught error may leave something on the stack Problem: Vim9: a caught error may leave something on the stack. Solution: Drop items from the stack if needed. (closes https://github.com/vim/vim/issues/7826)
author Bram Moolenaar <Bram@vim.org>
date Fri, 12 Feb 2021 21:45:04 +0100
parents 80a01c446853
children dba960dacd31
files src/testdir/test_vim9_script.vim src/version.c src/vim9execute.c
diffstat 3 files changed, 21 insertions(+), 1 deletions(-) [+]
line wrap: on
line diff
--- a/src/testdir/test_vim9_script.vim
+++ b/src/testdir/test_vim9_script.vim
@@ -556,6 +556,16 @@ def Test_try_catch_throw()
     n = 411
   endtry
   assert_equal(411, n)
+
+  var counter = 0
+  for i in range(4)
+    try
+      eval [][0]
+    catch
+    endtry
+    counter += 1
+  endfor
+  assert_equal(4, counter)
 enddef
 
 def Test_cnext_works_in_catch()
--- a/src/version.c
+++ b/src/version.c
@@ -751,6 +751,8 @@ static char *(features[]) =
 static int included_patches[] =
 {   /* Add new patch number below this line */
 /**/
+    2503,
+/**/
     2502,
 /**/
     2501,
--- a/src/vim9execute.c
+++ b/src/vim9execute.c
@@ -24,7 +24,8 @@
 
 // Structure put on ec_trystack when ISN_TRY is encountered.
 typedef struct {
-    int	    tcd_frame_idx;	// ec_frame_idx when ISN_TRY was encountered
+    int	    tcd_frame_idx;	// ec_frame_idx at ISN_TRY
+    int	    tcd_stack_len;	// size of ectx.ec_stack at ISN_TRY
     int	    tcd_catch_idx;	// instruction of the first catch
     int	    tcd_finally_idx;	// instruction of the finally block
     int	    tcd_caught;		// catch block entered
@@ -2561,6 +2562,7 @@ call_def_function(
 		    ++ectx.ec_trystack.ga_len;
 		    ++trylevel;
 		    trycmd->tcd_frame_idx = ectx.ec_frame_idx;
+		    trycmd->tcd_stack_len = ectx.ec_stack.ga_len;
 		    trycmd->tcd_catch_idx = iptr->isn_arg.try.try_catch;
 		    trycmd->tcd_finally_idx = iptr->isn_arg.try.try_finally;
 		    trycmd->tcd_caught = FALSE;
@@ -2632,6 +2634,12 @@ call_def_function(
 
 			if (trycmd->tcd_return)
 			    goto func_return;
+
+			while (ectx.ec_stack.ga_len > trycmd->tcd_stack_len)
+			{
+			    --ectx.ec_stack.ga_len;
+			    clear_tv(STACK_TV_BOT(0));
+			}
 		    }
 		}
 		break;